"""
    Contains a script to generate unit tests out of a file of sentences.
    Whenever you change your sentences file, you should regenerate these.
    
    To generate them (with Sentences in the same dir), call from command line:
        python generate_tests.py
    
    To run the tests, call from command line:
        python tests.py
        
    If you're getting a lot of skips, take a look with:
        python -m unittest discover . -v
"""
__author__="sdrobert"

from os.path import exists
from sys import exit
from q1utils import sanitizeAndValidateSentences
from re import compile as rec

if __name__=='__main__':
    
    PREFIX_FILE_NAME="unittest_prefix.py"
    SENTENCES_FILE_NAME="Sentences"
    TESTS_FILE_NAME = "tests.py"
    
    if not exists(SENTENCES_FILE_NAME):
        print("Can't generate sentences: \"{}\" doesn't exist.".format(
            SENTENCES_FILE_NAME))
        exit(1)
    
    preamble = None
    preamblePattern = rec('^%\s*(?P<givenName>\S+)\s+(?P<familyName>\S+)\s*,'
        '\s*(?P<loginID>\w+)\s*,\s*(?P<studentNumber>\d+)$')
    
    # get a hold of the preamble
    with open(SENTENCES_FILE_NAME,'r') as f:
        for line in f:
            sl = line.strip()
            if sl:
                match = preamblePattern.match(sl)
                if match:
                    preamble = sl
                    break
                else:
                    print("The first non-empty line in \"{}\" isn't in the "
                        "format \"%<givenName> <familyName>,<loginID>,"
                        "<studentNumber>\"".format(SENTENCES_FILE_NAME))
                    exit(1)
    if preamble is None:
        print('File "{}" is devoid of content'.format(SENTENCES_FILE_NAME))
    
    # sanitize and look for sentences
    sentencesValid,sentences = sanitizeAndValidateSentences(SENTENCES_FILE_NAME)
    if not sentencesValid:
        exit(1) # error messages output by function
    sentences = sentences.split('\n')
    if len(set(sentences)) != len(sentences):
        print('Some duplicate sentences exist in "{}". Not building tests.'
            ''.format(SENTENCES_FILE_NAME))
        exit(1)
    
    # get prefix
    if not exists(PREFIX_FILE_NAME):
        print('Prefix file "{}" missing'.format(PREFIX_FILE_NAME))
        exit(1)
    prefix = None
    with open(PREFIX_FILE_NAME,'r') as f:
        prefix = ''.join(filter(lambda x: x.strip(),
            [line.split('#')[0] for line in f.readlines()]))
    
    # actually write the file
    with open(TESTS_FILE_NAME,'w') as f:
        f.write(prefix)
        
        f.write('''
    PREAMBLE = '{}'
                '''.format(preamble))
        
        for sentence in sentences:
            shouldParse = True
            if sentence[0] == '*':
                sentence = sentence[1:]
                shouldParse = False
            f.write('''
    def test_{}(self):
        self.assert{}(self._runSentenceHelper('{}'),
            '{} "{}"')
                    '''.format(
                    'parse' + ''.join(sentence.title().split()), # camel case
                    shouldParse,
                    sentence,
                    'Could not parse' if shouldParse else 'Parsed',
                    sentence))
        
        f.write('''

if __name__=='__main__':
    unittest.main()
                ''')
    
    print('Wrote to ' + TESTS_FILE_NAME)
